#
# Jeckel Data Kit
# Based on Stone-D's SDToolKit
#
# CvJDataStorageUtils
# by Jeckel
# 
# 

from CvPythonExtensions import *
import CvUtil
import sys

#######SD Tool Kit#######

import SdToolKit
sdEcho         = SdToolKit.sdEcho
sdModInit      = SdToolKit.sdModInit
sdModLoad      = SdToolKit.sdModLoad
sdModSave      = SdToolKit.sdModSave
sdEntityInit   = SdToolKit.sdEntityInit
sdEntityExists = SdToolKit.sdEntityExists
sdGetEntity    = SdToolKit.sdGetEntity
sdGetVal       = SdToolKit.sdGetVal
sdSetVal       = SdToolKit.sdSetVal
sdDelVal       = SdToolKit.sdDelVal
sdEntityWipe   = SdToolKit.sdEntityWipe

###################################################
# globals

gc = CyGlobalContext()

############################################################################

###########################################################
# Here are the methods used to deal with SD ToolKit data. #
###########################################################

#########################################
# The init methods that set blank dicts #
#########################################

def initTypesDict(pUnit, dictInit=None):

        # Return is the unit is invalid
        if (pUnit == None) or (pUnit.isNone()) or (pUnit.isDead()):
                return False

        strUnitID = getDataIDStr(pUnit)

        if (dictInit != None) and (len(dictInit) >= 1):
                sdEntityInit("JUnitStatsMod", strUnitID, dictInit)
                return True

        # Create the UnitStats dictionary and set it
	sdEntityInit("JUnitStatsMod", strUnitID, {})
	return True

def initTypeStatsDict(pUnit, xType):

        # Return is the unit is invalid
        if (pUnit == None) or (pUnit.isNone()) or (pUnit.isDead()):
                return False

        strUnitID = getDataIDStr(pUnit)
        return sdSetVal("JUnitStatsMod", strUnitID, xType, {})
########################
# Verification methods #
########################

def hasTypesDict(pUnit):

        # Return is the unit is invalid
        if (pUnit == None) or (pUnit.isNone()):
                return False

        strUnitID = getDataIDStr(pUnit)

        if (not sdEntityExists("JUnitStatsMod", strUnitID)):
                initTypesDict(pUnit)

        return True

def hasTypeStatsDict(pUnit, xType):

        # Return immediately if the unit passed in is invalid
	if (not hasTypesDict(pUnit)):
		return False

	if (xType == None):
                return False

	strUnitID = getDataIDStr(pUnit)

        try:
                tempTypeDict = sdGetVal("JUnitStatsMod", strUnitID, xType)
                return True
        except:
                return False

def hasTypeStat(pUnit, xType, xStat):

        # Return immediately if the unit passed in is invalid
	if (not hasTypeStatsDict(pUnit, xType)):
                return False

	if (xStat == None):
                return False

	strUnitID = getDataIDStr(pUnit)

        statsTypeDict = sdGetVal("JUnitStatsMod", strUnitID, xType)

        try:
                tempStat = statsTypeDict[xStat]
                return True
        except KeyError:
                return False

###################
# The set methods #
###################

def setTypeStatsDict(pUnit, xType, dictStats=None):

        # Return immediately if the unit passed in is invalid
	if (not hasTypesDict(pUnit)):
		return False

	strUnitID = getDataIDStr(pUnit)

        # If tempDictStats is equal to None then dictStats is and empty dictionary
	if (dictStats == None):
                return initTypeStatsDict(pUnit, xType)

        return sdSetVal("JUnitStatsMod", strUnitID, xType, dictStats)

def setTypeStat(pUnit, xType, xStat, xStatValue=None):

        # Return immediately if the unit passed in is invalid
	if (not hasTypeStatsDict(pUnit, xType)):
		return False

	if (xStat == None):
                return False

        strUnitID = getDataIDStr(pUnit)
	dictUnitTypes = getTypesDict(pUnit)
	dictTypeStats = dictUnitTypes[xType]
	if (xStatValue == None):
                dictTypeStats[xStat] = 0
        else:
                dictTypeStats[xStat] = xStatValue

        return sdSetVal("JUnitStatsMod", strUnitID, xType, dictTypeStats)

######################
# The delete methods #
######################

def delTypesDict(pUnit):

        # Return immediately if the unit passed in is invalid
	if (not hasTypesDict(pUnit)):
		return True

	strUnitID = getDataIDStr(pUnit)

        return sdEntityWipe("JUnitStatsMod", strUnitID)

def delTypeStatsDict(pUnit, xType):

        # Return immediately if the unit passed in is invalid
	if (not hasTypeStatsDict(pUnit, xType)):
		return True

	strUnitID = getDataIDStr(pUnit)

        return sdDelVal("JUnitStatsMod", strUnitID, xType)

def delTypeStat(pUnit, xType, xStat):

        # Return immediately if the unit passed in is invalid
	if (not hasTypeStat(pUnit, xType, xStat)):
		return True

	strUnitID = getDataIDStr(pUnit)

	dictTypeStats = sdGetVal("JUnitStatsMod", strUnitID, xType)

	del dictTypeStats[xStat]

        return sdSetVal("JUnitStatsMod", strUnitID, xType, dictTypeStats)

###################
# The get methods #
###################

def getDataIDStr(pObj=None):

        # Return is the unit is invalid
        if (pObj == None) or (pObj.isNone()):
                return "XXXXXXXX_XXXXXXXX"

        return str(pObj.getOwner()) + "_" + str(pObj.getID())

def getUnitStatsStrType_StrStatList(pUnit, strDelim="_", bStatsTypeFirst=True):

        dictUnitStats = getTypesDict(pUnit)

        listStatsTypes = statsDict.keys()

        listStrType_StrStat = []

        for xType in listStatsTypes:
                listStatsTypeStats = getUnitStatsTypeStatsList(pUnit, xType)
                for xStat in listStatsTypeStats:
                        if (bStatsTypeFirst):
                                StrType_StrStat = str(xType) + strDelim + str(xStat)
                                listStrType_StrStat += StrType_StrStat
                                continue
                        StrStat_StrType = str(xStat) + strDelim + str(xType)
                        listStrStat_StrType += StrStat_StrType
                        continue

        if (bStatsTypeFirst):
                return listStrType_StrStat

        return listStrStat_StrType

def getTypesDict(pUnit):

        # Return immediately if the unit passed in is invalid
	if (not hasTypesDict(pUnit)):
                return False

	strUnitID = getDataIDStr(pUnit)

        dictTypes = sdGetEntity("JUnitStatsMod", strUnitID)

        return dictTypes

def getTypeStatsDict(pUnit, xType):

        # Return immediately if the unit passed in is invalid
	if (not hasTypeStatsDict(pUnit, xType)):
                return False

	strUnitID = getDataIDStr(pUnit)

        dictTypes = sdGetEntity("JUnitStatsMod", strUnitID)

        return dictTypes[xType]

def getTypeStat(pUnit, xType, xStat):

        # Return immediately if the unit passed in is invalid
	if (not hasTypeStat(pUnit, xType, xStat)):
                return None

	strUnitID = getDataIDStr(pUnit)

        dictTypes = sdGetEntity("JUnitStatsMod", strUnitID)
        dictStats = dictTypes[xType]

        return dictStats[xStat]

################################################################
# The following methods use the above methods to allow storage #
# of data in a global pool. This global pool is store under a  #
# non sensical DataIDStr which can be retrieved by passing no  #
# arguments or any None obj to getDataIDStr() which normally   #
# would takes a unit obj.                                      #
# Note that getDataIDStr() does no validation on unit          #
# obj it receives so do your checking before passing to it.    #
################################################################

def initGlobalDict():
        strGlobalID = getDataIDStr()

        # Create the Global Data Pool
	sdEntityInit("JUnitStatsMod", strGlobalID, {})
	return True

def hasGlobalDict():
        strGlobalID = getDataIDStr()

        if (not sdEntityExists("JUnitStatsMod", strGlobalID)):
                initGlobalDict()

        return True

def hasGlobalData(xKey):
        if (not hasGlobalDict()):
                return False

        strGlobalID = getDataIDStr()

        try:
                xValue = sdGetVal("JUnitStatsMod", strGlobalID, xKey)
                return True
        except:
                return False

def setGlobalData(xKey, xValue):
        if (not hasGlobalDict()):
                return False

        strGlobalID = getDataIDStr()

        return sdSetVal("JUnitStatsMod", strGlobalID, xKey, xValue)

def getGlobalData(xKey=None):

        strGlobalID = getDataIDStr()

        if (xKey == None) and (hasGlobalDict()):
                return sdGetEntity("JUnitStatsMod", strGlobalID)

        if (not hasGlobalData(xKey)):
                return None

        return sdGetVal("JUnitStatsMod", strGlobalID, xKey)













